open Gui

exception UnifyError
exception StructualError of string

(* var_t *)
module Var_e = struct
  type 'var_t t = V of string

  let map f v = match v with V(s) -> V(s)

  let search _ = false

  let unify f v1 v2 = match (v1, v2) with
      (V (s1), V (s2)) when s1 = s2 -> ()
    | _ -> raise (Unify_Error ("of Var")) 

  let draw v = match v with V(s) -> create_str s
end

module Var  = Fix.F (Var_e)

let var n = Var.make_fix (Var_e.V (n))

(* term_t *)
module Term_e = struct 
  type 'term_t t = Var of Var.fix_t
		 | Arrow of 'term_t * 'term_t 

  let map f tm = match tm with
      Var (v) -> Var (v)
    | Arrow (f1, f2) -> Arrow (f f1, f f2)

  let search tm = match tm with
      Var (v) -> Var.search v
    | Arrow (tm1, tm2) -> tm1 || tm2

  let unify f tm1 tm2 =  match (tm1, tm2) with
      (Var (v1), Var (v2)) -> Var.unify v1 v2
    | (Arrow (f1, f2), Arrow (f3, f4)) -> f f1 f3; f f2 f4
    | _ -> raise (Unify_Error "of Term")

  let draw tm = match tm with
      Var (v) -> Var.draw v
    | Arrow (fix1, fix2) -> 
	combineH [create_str "("; fix1; create_str "⊃"; fix2; create_str ")"]
end

module Term = Fix.F (Term_e)

let term_var v = Term.make_fix (Term_e.Var (v))
let term_arrow e1 e2 = Term.make_fix (Term_e.Arrow (e1, e2))

(* 推論規則を定義する *)

(* T_S *)
let t_s () =
  let tm1 = Term.make_meta "t" in
  let tm2 = Term.make_meta "t" in
  let tm3 = Term.make_meta "t" in
    Axiom(term_arrow (term_arrow tm1 (term_arrow tm2 tm3))
		     (term_arrow (term_arrow tm1 tm2)
				 (term_arrow tm1 tm3)))

(* T_K *)
let t_k () =
  let tm1 = Term.make_meta "t" in
  let tm2 = Term.make_meta "t" in
    Axiom(term_arrow tm1 (term_arrow tm2 tm1))

(* T_MP *)
let t_mp () = 
  let tm1 = Term.make_meta "t" in
  let tm2 = Term.make_meta "t" in
    Infer(tm2,
	  [Infer(tm1, []);
	   Infer(term_arrow tm1 tm2,
		 [])])

(* 登録される推論規則 *)
let infer_rule_list = 
  [("S", t_s); ("K", t_k); ("MP", t_mp)]
